"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ngsscServeWebpackBrowser = void 0;
const architect_1 = require("@angular-devkit/architect");
const build_angular_1 = require("@angular-devkit/build-angular");
const index_1 = require("../ngsscbuild/index");
/**
 * Ngssc wrapper for the Angular Webpack development server builder.
 * @param options Dev Server options.
 * @param context The build context.
 * @param transforms A map of transforms that can be used to hook into some logic (such as
 *     transforming webpack configuration before passing it to webpack).
 *
 * @experimental Direct usage of this function is considered experimental.
 */
function ngsscServeWebpackBrowser(options, context, transforms = {}) {
    transforms.indexHtml = (content) => __awaiter(this, void 0, void 0, function* () {
        const ngsscContext = yield (0, index_1.detectVariables)(context, options.searchPattern);
        const ngssc = (0, index_1.buildNgssc)(ngsscContext, options);
        const populatedVariables = populateVariables(ngssc.environmentVariables);
        context.logger.info(`Populated environment variables (Variant: ${ngssc.variant})\n${Object.entries(populatedVariables).map(([key, value]) => `  ${key}: ${value}`)}`);
        const iife = generateIife(ngssc.variant, populatedVariables);
        return insertIife(content, iife);
    });
    return (0, build_angular_1.executeDevServerBuilder)(options, context, transforms);
}
exports.ngsscServeWebpackBrowser = ngsscServeWebpackBrowser;
function populateVariables(variables) {
    const populatedVariables = {};
    variables.forEach((v) => (populatedVariables[v] = v in process.env ? process.env[v] || '' : null));
    return populatedVariables;
}
function generateIife(variant, populatedVariables) {
    const iife = variant === 'NG_ENV'
        ? `(function(self){self.NG_ENV=${JSON.stringify(populatedVariables)};})(window)`
        : `(function(self){self.process=${JSON.stringify({ env: populatedVariables })};})(window)`;
    return `<!--ngssc--><script>${iife}</script><!--/ngssc-->`;
}
function insertIife(fileContent, iife) {
    if (/<!--ngssc-->[\w\W]*<!--\/ngssc-->/.test(fileContent)) {
        return fileContent.replace(/<!--ngssc-->[\w\W]*<!--\/ngssc-->/, iife);
    }
    else if (/<!--\s*CONFIG\s*-->/.test(fileContent)) {
        return fileContent.replace(/<!--\s*CONFIG\s*-->/, iife);
    }
    else if (fileContent.includes('</title>')) {
        return fileContent.replace('</title>', `</title>${iife}`);
    }
    else {
        return fileContent.replace('</head>', `${iife}</head>`);
    }
}
exports.default = (0, architect_1.createBuilder)(ngsscServeWebpackBrowser);
//# sourceMappingURL=index.js.map