"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildNgssc = exports.detectVariables = exports.detectVariablesAndBuildNgsscJson = exports.ngsscBuild = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
const architect_1 = require("@angular-devkit/architect");
const glob = __importStar(require("glob"));
const variable_detector_1 = require("./variable-detector");
function ngsscBuild(options, context) {
    return __awaiter(this, void 0, void 0, function* () {
        const buildTarget = (0, architect_1.targetFromTargetString)(options.buildTarget || options.browserTarget);
        const rawBuilderOptions = yield context.getTargetOptions(buildTarget);
        const builderName = yield context.getBuilderNameForTarget(buildTarget);
        const builderOptions = yield context.validateOptions(rawBuilderOptions, builderName);
        const scheduledTarget = yield context.scheduleTarget(buildTarget);
        const result = yield scheduledTarget.result;
        if (!result.success) {
            const buildConfig = buildTarget.configuration ? `:${buildTarget.configuration}` : '';
            context.logger.warn(`ngssc: Failed build of ${buildTarget.app}:${buildTarget.target}${buildConfig}. Skipping ngssc build.`);
            return result;
        }
        yield detectVariablesAndBuildNgsscJson(options, builderOptions, context, false, builderName !== '@angular-devkit/build-angular:application'
            ? undefined
            : 'server' in builderOptions && builderOptions.server
                ? 'server'
                : 'browser-only');
        return result;
    });
}
exports.ngsscBuild = ngsscBuild;
function detectVariablesAndBuildNgsscJson(options, builderOptions, context, multiple = false, applicationBuilderVariant = undefined) {
    return __awaiter(this, void 0, void 0, function* () {
        const ngsscContext = yield detectVariables(context, options.searchPattern);
        let outputPath = (0, path_1.join)(context.workspaceRoot, builderOptions.outputPath);
        const ngssc = buildNgssc(ngsscContext, options, builderOptions, multiple, applicationBuilderVariant);
        const browserOutputPath = (0, path_1.join)(outputPath, 'browser');
        if (applicationBuilderVariant === 'browser-only' && (0, fs_1.existsSync)(browserOutputPath)) {
            outputPath = browserOutputPath;
        }
        (0, fs_1.writeFileSync)((0, path_1.join)(outputPath, 'ngssc.json'), JSON.stringify(ngssc, null, 2), 'utf8');
    });
}
exports.detectVariablesAndBuildNgsscJson = detectVariablesAndBuildNgsscJson;
function detectVariables(context, searchPattern) {
    return __awaiter(this, void 0, void 0, function* () {
        const projectName = context.target && context.target.project;
        if (!projectName) {
            throw new Error('The builder requires a target.');
        }
        const projectMetadata = yield context.getProjectMetadata(projectName);
        const sourceRoot = projectMetadata.sourceRoot;
        const defaultSearchPattern = sourceRoot
            ? `${sourceRoot}/**/environments/environment*.ts`
            : '**/environments/environment*.ts';
        const detector = new variable_detector_1.VariableDetector(context.logger);
        const typeScriptFiles = yield glob.glob(searchPattern || defaultSearchPattern, {
            absolute: true,
            cwd: context.workspaceRoot,
            ignore: ['**/node_modules/**', '**/*.spec.ts', '**/*.d.ts'],
        });
        let ngsscContext = null;
        for (const file of typeScriptFiles) {
            const fileContent = (0, fs_1.readFileSync)(file, 'utf8');
            const innerNgsscContext = detector.detect(fileContent);
            if (!innerNgsscContext.variables.length) {
                continue;
            }
            else if (!ngsscContext) {
                ngsscContext = innerNgsscContext;
                continue;
            }
            if (ngsscContext.variant !== innerNgsscContext.variant) {
                context.logger.info(`ngssc: Detected conflicting variants (${ngsscContext.variant} and ${innerNgsscContext.variant}) being used`);
            }
            ngsscContext.variables.push(...innerNgsscContext.variables.filter((v) => !ngsscContext.variables.includes(v)));
        }
        if (!ngsscContext) {
            return { variant: 'process', variables: [] };
        }
        context.logger.info(`ngssc: Detected variant '${ngsscContext.variant}' with variables ` +
            `'${ngsscContext.variables.join(', ')}'`);
        return ngsscContext;
    });
}
exports.detectVariables = detectVariables;
function buildNgssc(ngsscContext, options, builderOptions, multiple = false, applicationBuilderVariant = undefined) {
    return {
        environmentVariables: [
            ...ngsscContext.variables,
            ...(options.additionalEnvironmentVariables || []),
        ],
        filePattern: options.filePattern ||
            extractFilePattern(builderOptions, multiple, applicationBuilderVariant),
        variant: ngsscContext.variant,
    };
}
exports.buildNgssc = buildNgssc;
function extractFilePattern(builderOptions, multiple, applicationBuilderVariant = undefined) {
    if (builderOptions && applicationBuilderVariant === 'server') {
        return '**/index{.,.server.}html';
    }
    const index = builderOptions === null || builderOptions === void 0 ? void 0 : builderOptions.index;
    let result = '**/index.html';
    if (!index || typeof index === 'boolean') {
        return result;
    }
    else if (typeof index === 'string') {
        result = (0, path_1.basename)(index);
    }
    else if (index.output) {
        result = (0, path_1.basename)(index.output);
    }
    else {
        result = (0, path_1.basename)(index.input);
    }
    return multiple && !result.startsWith('*') ? `**/${result}` : result;
}
exports.default = (0, architect_1.createBuilder)(ngsscBuild);
//# sourceMappingURL=index.js.map